from loguru import logger
import json
import asyncio 
import aiohttp
import requests
import time


class CaptchaAi:
    def __init__(self, token:str, async_:bool=False, threads:int=5):
        '''
        `token`  : is your <API_KEY> from captchaai.com
        `async_` : default False
        `threads`: default 5, working only with async mode 
        ''' 
        self.token = token
        self.async_=async_ 
        if async_:
            self.threads = threads
        self.url_solve_picture = 'https://ocr.captchaai.com/in.php'
        self.url_result = 'https://ocr.captchaai.com/res.php'

    def solve_picture(self, image:bytes, timeout:int=5, retries:int=3, **kwargs):
        '''
        `timeout`:  default 5
        `retries`:  default 3 How many retries to solve captcha.
        `phrase`:   0 - captcha contains one word
                    1 - captcha contains two or more words
        `regsense`: 0 - captcha in not case sensitive
                    1 - captcha is case sensitive
        `numeric`: 0 - not specified
                    1 - captcha contains only numbers
                    2 - captcha contains only letters
                    3 - captcha contains only numbers OR only letters
                    4 - captcha contains both numbers AND letters
        `calc`:     0 - not specified
                    1 - captcha requires calculation (e.g. type the result 4 + 8 = )
        `min_len`:  0 - not specified
                    1..20 - minimal number of symbols in captcha
        `max_len`:  0 - not specified
                    1..20 - maximal number of symbols in captcha
        `language`: 0 - not specified  
                    1 - Cyrillic captcha   
                    2 - Latin captcha  
        `lang`:     Language code. [See the list of supported languages](https://captchaai.com/api-docs.php#language).

        parameters `json`, `method` - doesn't support!
        '''
        data = self._get_picture_data(kwargs)
        if self.async_:
            return self._async_solve_picture(image, timeout, retries, data)
        return self._sync_solve_picture(image, timeout, retries, data)            


    def _get_picture_data(self, kwargs):
        return {'key':self.token,
                'method':'base64',
                'json':1, **kwargs}
    

    def _get_picture_params(self, key):
        return {'key':self.token,
                'action':'get',
                'json':1, 
                'id':key}

    @logger.catch
    def _sync_get_key_picture(self, image:bytes, data:dict) -> str:
        logger.info(f'url = {self.url_solve_picture}\n{data = }')
        response = requests.post(self.url_solve_picture, 
                                 params=data, 
                                 files={'file':image})
        if response.status_code == 200 and response.json().get('status') == 1:
            return response.json().get('request')
        else: 
            raise Exception(f'Проблема с отправкой:\n'
                            f'{response.status_code = }\n{response.content}')


    @logger.catch
    def _sync_get_result(self, retries:int, timeout:int, key:str):
        params = self._get_picture_params(key)
        for _ in range(retries):
            time.sleep(timeout)
            response = requests.get(self.url_result,params=params)
            try:
                if response.status_code == 200 and response.json().get('status') == 1:
                    return response.json().get('request')
                else: 
                    logger.info(f'Проблемка с получение {response.content}')
            except Exception as e:
                logger.info(response.content)
                logger.warning(e)
        raise Exception(f'Не удалось получить ответ с трёх попыток {response.content = }')


    def _sync_solve_picture(self, image:bytes, timeout:int, retries:int, data: dict):
        key = self._sync_get_key_picture(image, data)
        logger.debug(f'{key = }')
        return self._sync_get_result(retries, timeout, key)



    async def _async_get_result(self, retries:int, timeout:int, key:str):
        async with aiohttp.ClientSession() as session:
            params = self._get_picture_params(key)
            for _ in range(retries):
                logger.debug(f'sleep')
                await asyncio.sleep(timeout)
                async with session.get(self.url_result, params=params) as resp:
                    if resp.status == 200:
                        result = await resp.json(content_type='text/html')
                        if result.get('status') != 1: 
                            raise Exception(f'Что то пошло не так {await resp.text()}')
                        return result.get('request')
                    else: 
                        logger.info(f'Проблемы при получении результата {resp.status} {await resp.text()}')

    @logger.catch
    async def _async_get_key_picture(self, image:bytes, data: dict):
        async with aiohttp.ClientSession() as session:
            form_data = aiohttp.FormData()
            for key, value in data.items():
                form_data.add_field(key, str(value))
            form_data.add_field('file', image)
            async with session.post(self.url_solve_picture, data=form_data) as resp:
                try:
                    if resp.status == 200: 
                        result = await resp.json(content_type='text/html')
                        logger.info(result)                
                        return result.get('request')
                    else: 
                        raise Exception(f'Не получилось послать... {resp.status = }, {await resp.text() = }')
                except Exception as e: 
                     logger.info(e)
                     logger.info(f'{await resp.text()}')



    async def _async_solve_picture(self, image:bytes, timeout:int, retries:int, data: dict):
        key = await self._async_get_key_picture(image, data)
        result = await self._async_get_result(retries, timeout, key)
        return result


from datetime import date, timedelta, datetime
import requests
from loguru import logger
import pandas as pd 
import json
from pydantic import BaseModel 


class ProxyManager:
    def __init__(self, token:int, url: str):
        '''
        url - адрес по которому посылать запрос для получения списка актуальных прокси
        token - токен для опознания на сервере
        '''
        self.url = url 
        self.token = token 
        self.types = {'string', 'dict[str,str]','playwright'}

    
    def get(self, request_type='string', expire:str | date | datetime=None):
        '''
        Возвращает список актуальных прокси    

        type: str : ['string', 'dict[str,str]','playwright']
        expire: str | date | datetime 

        ---
        `type` может быть следующих значений:
          - 'string', вернет в формате:   
            [{"http" : "http://user:pass@127.0.0.1:8000", "https":"https://user:pass@127.0.0.1:8000}, ...]
          - 'dict[str,str] вернет в формате:
            [{"server":"http://127.0.0.1:8000","username":"user","password":"pass"}...]
          - 'playwright'
            [{"proxy":{
              "server":"http://127.0.0.1:8000","username":"user","password":"pass"}
              },...]
        ---

        `expire` - до какой даты, если не указано, то вернет, те что работают до завтра
        передавать либо объект date | datetime, либо строку формата : "2023-02-15"        
        '''
        if request_type not in self.types:
            raise ValueError(f'type must be {self.types}')
        expire = self._check_expire(expire)
        return self._request(expire, request_type)


    def get_full(self, path:str = None):
        '''
        Возвращает все прокси которые есть на сервере
        '''
        resp = requests.get(url=f'{self.url}/full/', params={'token':self.token})
        if resp.status_code == 200:
            if path is None:
                logger.info(json.dumps(resp.json().get('data'), indent=4))
            else:
                pd.DataFrame(resp.json().get('data')).to_csv(path, index=False)
        else: # status_code != 200
            raise Exception(f'[{resp.status_code}] {resp.content}')


    def post(self, data: list[dict]=None, path:str=None):
        '''
        Опубликовать новые прокси
        Указывается либо `path` либо `data`, вместе нельзя и ничего указывать нельзя!

        `path`:str - Путь до файла (формат xlsx, csv), проверить на наличие соответствующих колонок  

        `data`:list[dict] - Список словарей, должен быть следующего формата:

        ```python
        data = [
        {"server":"127.0.0.1"}, 
        {"username":"user"},
        {"password":"pass"}, 
        {"port":8000},
        {"expire":"2023-02-23"} # Дата до которой прокси работает
        {"service":"example.service.ru"} # Опционально
        ]
        proxies = GetterProxy(token)
        proxies.post(data)
        ```
        '''
        if data is None and path is not None: 
            data = self._get_data_to_post_request(path)
        elif data is not None and path is None: 
            [_PostProxy(datum) for datum in data]
        else: 
            raise ValueError(f'you can set data OR path.')
        [_PostProxy(**datum) for datum in data]
        self._request_post(data)

    def delete(self, id):
        '''Удаляет прокси с сервера по id'''
        resp = requests.delete(self.url, params={'token':self.token, 'id':id})
        logger.info(resp.status_code)
        if resp.status_code == 200 or resp.status_code == 201:
            logger.info(f'Succesful delete {resp.json()}')
        else: 
            raise Exception(f'[{resp.status_code}] {resp.json()}')
        
    def update(self, data:dict[str,str|int]):
        '''
        Обновляет прокси на сервере, пример словаря:
        ```python
        data = {
            "id":4, 
            # тут параметры которые надо изменить, например: 
            "service":"new_service",
            ...
        }
        ```
        '''
        resp = requests.put(url=self.url, params={'token':self.token}, data=json.dumps(data))
        if resp.status_code == 200 or resp.status_code == 201:
            logger.info(f'Succesful update {resp.json()}')
        else: 
            raise Exception(f'[{resp.status_code}] {resp.json()}')

    @staticmethod
    def _get_data_to_post_request(path: str):
        if path is not None: 
            type_fp = path.split('.')[-1]
        if type_fp == 'csv':
            data = pd.read_csv(path).to_dict('records')
        elif type_fp == 'xlsx':
            data = pd.read_excel(path, index_col=False).to_dict('records')
        else: # type_fp is not 'csv' and 'xlsx'
            raise TypeError(f'type file can be only `.xlsx` or `.csv`')
        return data
    
    def _request_post(self, data):
        data = {
            'token':self.token,
            'data': data
        }
        resp = requests.post(self.url, data=json.dumps(data))
        if resp.status_code == 200 or resp.status_code == 201:
            logger.info(resp.content)
        else:
            raise Exception(f'[{resp.status_code}] {resp.json()}')

    def _request(self, expire, request_type):
        params = self._make_params(expire, request_type)
        resp =  requests.get(self.url, params=params)
        if resp.status_code == 200: 
            logger.info(f'Получены прокси: {len(resp.json().get("data"))} шт')
            return resp.json().get('data')
        elif resp.status_code == 204:
            raise Exception('Нет прокси актуальных...') 
        else:
            raise Exception(f'[{resp.status_code}] {resp.content}')


    @staticmethod
    def _check_expire(expire):
        if isinstance(expire, date | datetime):
            return expire.strftime('%Y-%m-%d')
        if expire is None:
            return (date.today() + timedelta(days=1)).strftime('%Y-%m-%d')
        try: 
            return datetime.strptime(expire, '%Y-%m-%d').strftime('%Y-%m-%d')
        except:
            raise ValueError(f'expire must be in format 2023-02-15')      


    def _make_params(self, expire, request_type):
        return {
            'token':self.token,
            'expire': expire,
            'type':request_type
        }


class _PostProxy(BaseModel):
    server: str 
    port: int 
    password: str
    username: str 
    expire: str | date | datetime
    service: str | None = None 



from setuptools import setup, find_packages

with open('README.md', 'r', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="mangust228",
    version="0.6",
    description="Library for take current proxy from API",
    author="mangust228",
    author_email="bacek.mangust@gmail.com",
    packages=find_packages(),
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/mangustik228/proxy_manager',
    install_requires=[
        'requests',
        'pandas',
        'pydantic',
        'loguru'
    ],
    classifiers=[
    'Programming Language :: Python :: 3',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent'
    ],
    python_requires = '>=3.8'
)